import argparse
import bz2
import os

from sklearn.datasets import fetch_20newsgroups
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.model_selection import train_test_split


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-dp', '--data_path', type=str, default='news20_data')
    parser.add_argument('-dn', '--data_name', type=str, default='news20')
    parser.add_argument('-sl', '--split_list', type=list, default=['train', 'test'])
    parser.add_argument('-vs', '--val_size', type=list, default=0.2)
    args = parser.parse_args()
    return args


def split2name(split):
    _split2name = {'train': 'train', 'valid': 'val', 'test': 'test'}
    return _split2name[split]


def get_texts(dataset):
    return [' '.join(text.split()) for text in dataset.data]


def get_labels(dataset):
    return list(map(str, dataset.target))


def main():
    # args
    args = get_args()
    os.makedirs(args.data_path, exist_ok=True)

    # process
    processed_data = {}
    for split in args.split_list:
        dataset = fetch_20newsgroups(subset=split)
        texts = get_texts(dataset)
        labels = get_labels(dataset)
        assert len(texts) == len(labels)
        print(f'{args.data_name} ({split}): num_instance = {len(texts)}')
        processed_data[split] = {'text': texts, 'labels': labels}

    # tfidf (train/test)
    vectorizer = TfidfVectorizer()
    vectorizer.fit(processed_data['train']['text'])

    for split in args.split_list:
        split_file = f'{args.data_name}_tfidf_{split2name(split)}.svm.bz2'
        split_path = os.path.join(args.data_path, split_file)

        vectors = vectorizer.transform(processed_data[split]['text'])
        vectors = vectors.tolil()

        with bz2.open(split_path, 'wb') as f:
            for labels, feature, index in zip(processed_data[split]['labels'], vectors.data, vectors.rows):
                assert len(feature) == len(index)
                labels_str = labels.replace(' ', ',')
                feature_str = ' '.join([f'{i+1}:{f}' for f, i in zip(feature, index)])
                f.write(f'{labels_str} {feature_str}\n'.encode('utf-8'))

    # raw texts (train/valid/test)
    processed_data['valid'] = {}
    train_text, valid_text, train_labels, valid_labels = train_test_split(
        processed_data['train']['text'],
        processed_data['train']['labels'],
        test_size=args.val_size,
        random_state=42
    )
    processed_data['train'] = {'text': train_text, 'labels': train_labels}
    processed_data['valid'] = {'text': valid_text, 'labels': valid_labels}

    for split in processed_data:
        split_file = f'{args.data_name}_raw_texts_{split2name(split)}.txt.bz2'
        split_path = os.path.join(args.data_path, split_file)
        with bz2.open(split_path, 'wb') as f:
            for text, label in zip(processed_data[split]['text'], processed_data[split]['labels']):
                formatted_instance = '\t'.join([label, text])
                f.write(f'{formatted_instance}\n'.encode('utf-8'))


if __name__ == '__main__':
    main()
